# Copyright (C) 2006-2018 Istituto Italiano di Tecnologia (IIT)
# Copyright (C) 2006-2010 RobotCub Consortium
# All rights reserved.
#
# This software may be modified and distributed under the terms of the
# BSD-3-Clause license. See the accompanying LICENSE file for details.

include(GNUInstallDirs)
include(CMakePackageConfigHelpers)
include(YarpBackupVariable)

set(YARP_BINDINGS "${CMAKE_SOURCE_DIR}/bindings")
set(YARP_CMAKECONFIG_DIR "${CMAKE_BINARY_DIR}")

set(YARP_DEFAULT_FIND_COMPONENTS OS
                                 sig
                                 dev)
if(TARGET YARP::YARP_math)
  list(APPEND YARP_DEFAULT_FIND_COMPONENTS math)
endif()

configure_package_config_file("${CMAKE_CURRENT_LIST_DIR}/template/YARPConfig.cmake.in"
                              "${CMAKE_BINARY_DIR}/YARPConfig.cmake"
                              INSTALL_DESTINATION "${CMAKE_BINARY_DIR}"
                              INSTALL_PREFIX "${CMAKE_BINARY_DIR}"
                              PATH_VARS YARP_BINDINGS
                                        YARP_MODULE_DIR
                                        YARP_IDL_BINARY_HINT
                                        # YARP_INSTALL_PREFIX is empty for build tree
                                        # The following variables are used both for the relative
                                        # and absolute directory in YARPConfig.cmake.in using
                                        # @VAR@ for relative path or @PACKAGE_VAR for absolute.
                                        YARP_DATA_INSTALL_DIR
                                        YARP_CONFIG_INSTALL_DIR
                                        YARP_PLUGIN_MANIFESTS_INSTALL_DIR
                                        YARP_MODULES_INSTALL_DIR
                                        YARP_APPLICATIONS_INSTALL_DIR
                                        YARP_TEMPLATES_INSTALL_DIR
                                        YARP_APPLICATIONS_TEMPLATES_INSTALL_DIR
                                        YARP_MODULES_TEMPLATES_INSTALL_DIR
                                        YARP_CONTEXTS_INSTALL_DIR
                                        YARP_ROBOTS_INSTALL_DIR
                                        YARP_STATIC_PLUGINS_INSTALL_DIR
                                        YARP_DYNAMIC_PLUGINS_INSTALL_DIR
                                        YARP_QML2_IMPORT_DIR
                                        YARP_CMAKECONFIG_DIR)

write_basic_package_version_file(${CMAKE_BINARY_DIR}/YARPConfigVersion.cmake
                                 VERSION ${YARP_VERSION_SHORT}
                                 COMPATIBILITY SameMajorVersion)


# YARPTargets.cmake (build tree)
export(EXPORT YARP
       NAMESPACE YARP::
       FILE ${CMAKE_BINARY_DIR}/YARPTargets.cmake)

if(NOT BUILD_SHARED_LIBS)
  configure_file("${CMAKE_CURRENT_LIST_DIR}/template/YARPTargetsStatic.cmake.in"
                 "${CMAKE_BINARY_DIR}/YARPTargetsStatic.cmake"
                 @ONLY)
endif()

if(NOT CMAKE_MINIMUM_REQUIRED_VERSION VERSION_LESS 3.7)
  message(AUTHOR_WARNING "Searching for packages in \"lib/cmake\" is supported on Windows since CMake 3.7. You can remove this check")
endif()
if(WIN32)
  set(YARP_CMAKE_DESTINATION CMake)
else()
  set(YARP_CMAKE_DESTINATION "${CMAKE_INSTALL_LIBDIR}/cmake/YARP")
endif()

# Save variables for later
yarp_backup_variable(YARP_BINDINGS)
yarp_backup_variable(YARP_MODULE_DIR)
yarp_backup_variable(YARP_IDL_BINARY_HINT)
yarp_backup_variable(YARP_INSTALL_PREFIX)

# Set up a configuration file for installed use of YARP
set(YARP_BINDINGS "${CMAKE_INSTALL_FULL_DATADIR}/yarp/bindings")
set(YARP_MODULE_DIR "${CMAKE_INSTALL_FULL_DATADIR}/yarp/cmake")
set(YARP_IDL_BINARY_HINT "${CMAKE_INSTALL_FULL_BINDIR}")
set(YARP_INSTALL_PREFIX "${CMAKE_INSTALL_PREFIX}")
set(YARP_CMAKECONFIG_DIR "${YARP_CMAKE_DESTINATION}/..")

configure_package_config_file("${CMAKE_CURRENT_LIST_DIR}/template/YARPConfig.cmake.in"
                              "${CMAKE_BINARY_DIR}/YARPConfigForInstall.cmake"
                              INSTALL_DESTINATION ${YARP_CMAKE_DESTINATION}
                              PATH_VARS YARP_BINDINGS
                                        YARP_MODULE_DIR
                                        YARP_IDL_BINARY_HINT
                                        YARP_INSTALL_PREFIX
                                        # The following variables are used both for the relative
                                        # and absolute directory in YARPConfig.cmake.in using
                                        # @VAR@ for relative path or @PACKAGE_VAR for absolute.
                                        YARP_DATA_INSTALL_DIR
                                        YARP_CONFIG_INSTALL_DIR
                                        YARP_PLUGIN_MANIFESTS_INSTALL_DIR
                                        YARP_MODULES_INSTALL_DIR
                                        YARP_APPLICATIONS_INSTALL_DIR
                                        YARP_TEMPLATES_INSTALL_DIR
                                        YARP_APPLICATIONS_TEMPLATES_INSTALL_DIR
                                        YARP_MODULES_TEMPLATES_INSTALL_DIR
                                        YARP_CONTEXTS_INSTALL_DIR
                                        YARP_ROBOTS_INSTALL_DIR
                                        YARP_STATIC_PLUGINS_INSTALL_DIR
                                        YARP_DYNAMIC_PLUGINS_INSTALL_DIR
                                        YARP_QML2_IMPORT_DIR
                                        YARP_CMAKECONFIG_DIR)
install(FILES ${CMAKE_BINARY_DIR}/YARPConfigForInstall.cmake
        RENAME YARPConfig.cmake
        COMPONENT configuration
        DESTINATION ${YARP_CMAKE_DESTINATION})
install(FILES ${CMAKE_BINARY_DIR}/YARPConfigVersion.cmake
        COMPONENT configuration
        DESTINATION ${YARP_CMAKE_DESTINATION})

# Restore variables to their original value
yarp_restore_variable(YARP_MODULE_DIR)
yarp_restore_variable(YARP_IDL_BINARY_HINT)
yarp_restore_variable(YARP_BINDINGS)
yarp_restore_variable(YARP_INSTALL_PREFIX)


# YARPTargets.cmake (installed)
install(EXPORT YARP
        NAMESPACE YARP::
        DESTINATION ${YARP_CMAKE_DESTINATION}
        FILE YARPTargets.cmake)

if(NOT BUILD_SHARED_LIBS)
  configure_file("${CMAKE_CURRENT_LIST_DIR}/template/YARPTargetsStatic.cmake.in"
                 "${CMAKE_BINARY_DIR}/YARPTargetsStaticForInstall.cmake"
                 @ONLY)
  install(FILES ${CMAKE_BINARY_DIR}/YARPTargetsStaticForInstall.cmake
          RENAME YARPTargetsStatic.cmake
          COMPONENT configuration
          DESTINATION ${YARP_CMAKE_DESTINATION})
endif()
